Compare commits

..

No commits in common. "1587a40514932dfb5309e06f891d27a42151963a" and "8a67e8712a54d81aaa4b56f1915565c8c264daa5" have entirely different histories.

2 changed files with 83 additions and 91 deletions

View File

@ -1,83 +1,78 @@
From 54ad3a9005a127a957bf081b0be765dcaff18c5d Mon Sep 17 00:00:00 2001
From 4d55356e918fd75a07d05db3ea28798a016405a4 Mon Sep 17 00:00:00 2001
From: Alex Denes <caskd@redxen.eu>
Date: Thu, 10 Jul 2025 12:01:29 +0000
Subject: [PATCH] Implement color manager
---
Makefile | 9 ++++++++-
config.def.h | 24 ++++++++++++++++++++----
dwl.c | 51 +++++++++++++++++++++++++++++++++++++++++++++++++++
3 files changed, 79 insertions(+), 5 deletions(-)
Makefile | 6 +++++-
config.def.h | 8 +++++---
config.mk | 4 ++--
dwl.c | 50 ++++++++++++++++++++++++++++++++++++++++++++++++++
4 files changed, 62 insertions(+), 6 deletions(-)
diff --git a/Makefile b/Makefile
index 3981fbb..190f29c 100644
index 578194f..570d5f5 100644
--- a/Makefile
+++ b/Makefile
@@ -22,7 +22,8 @@ dwl: dwl.o util.o
@@ -21,7 +21,8 @@ dwl: dwl.o util.o
$(CC) dwl.o util.o $(DWLCFLAGS) $(LDFLAGS) $(LDLIBS) -o $@
dwl.o: dwl.c client.h config.h config.mk cursor-shape-v1-protocol.h \
ext-image-copy-capture-v1-protocol.h \
pointer-constraints-unstable-v1-protocol.h wlr-layer-shell-unstable-v1-protocol.h \
- wlr-output-power-management-unstable-v1-protocol.h xdg-shell-protocol.h
+ wlr-output-power-management-unstable-v1-protocol.h xdg-shell-protocol.h \
+ color-management-v1-protocol.h color-representation-v1-protocol.h
+ color-management-v1-protocol.h
util.o: util.c util.h
# wayland-scanner is a tool which generates C headers and rigging for Wayland
@@ -43,6 +44,12 @@ pointer-constraints-unstable-v1-protocol.h:
@@ -39,6 +40,9 @@ pointer-constraints-unstable-v1-protocol.h:
wlr-layer-shell-unstable-v1-protocol.h:
$(WAYLAND_SCANNER) enum-header \
protocols/wlr-layer-shell-unstable-v1.xml $@
+color-management-v1-protocol.h:
+ $(WAYLAND_SCANNER) enum-header \
+ $(WAYLAND_PROTOCOLS)/staging/color-management/color-management-v1.xml $@
+color-representation-v1-protocol.h:
+ $(WAYLAND_SCANNER) enum-header \
+ $(WAYLAND_PROTOCOLS)/staging/color-representation/color-representation-v1.xml $@
wlr-output-power-management-unstable-v1-protocol.h:
$(WAYLAND_SCANNER) server-header \
protocols/wlr-output-power-management-unstable-v1.xml $@
diff --git a/config.mk b/config.mk
index eb08a05..f641d04 100644
--- a/config.mk
+++ b/config.mk
@@ -8,8 +8,8 @@ PREFIX = /usr/local
MANDIR = $(PREFIX)/share/man
DATADIR = $(PREFIX)/share
-WLR_INCS = `$(PKG_CONFIG) --cflags wlroots-0.19`
-WLR_LIBS = `$(PKG_CONFIG) --libs wlroots-0.19`
+WLR_INCS = `$(PKG_CONFIG) --cflags wlroots-0.20`
+WLR_LIBS = `$(PKG_CONFIG) --libs wlroots-0.20`
# Allow using an alternative wlroots installation
# This has to have all the includes required by wlroots, e.g:
diff --git a/dwl.c b/dwl.c
index 8101ffa..4c3f12a 100644
index 12f441e..39812ce 100644
--- a/dwl.c
+++ b/dwl.c
@@ -4,6 +4,7 @@
@ -88,26 +83,25 @@ index 8101ffa..4c3f12a 100644
#include <math.h>
#include <signal.h>
#include <stdio.h>
@@ -18,6 +19,8 @@
@@ -18,6 +19,7 @@
#include <wlr/render/wlr_renderer.h>
#include <wlr/types/wlr_alpha_modifier_v1.h>
#include <wlr/types/wlr_compositor.h>
+#include <wlr/types/wlr_color_management_v1.h>
+#include <wlr/types/wlr_color_representation_v1.h>
#include <wlr/types/wlr_cursor.h>
#include <wlr/types/wlr_cursor_shape_v1.h>
#include <wlr/types/wlr_data_control_v1.h>
@@ -219,6 +222,9 @@ typedef struct {
@@ -215,6 +217,9 @@ typedef struct {
const Layout *lt;
enum wl_output_transform rr;
int x, y;
+
+ struct wlr_output_image_description *image_desc;
+ int hdr;
+ uint32_t render_format;
} MonitorRule;
typedef struct {
@@ -386,6 +392,7 @@ static struct wlr_virtual_keyboard_manager_v1 *virtual_keyboard_mgr;
@@ -382,6 +387,7 @@ static struct wlr_virtual_keyboard_manager_v1 *virtual_keyboard_mgr;
static struct wlr_virtual_pointer_manager_v1 *virtual_pointer_mgr;
static struct wlr_cursor_shape_manager_v1 *cursor_shape_mgr;
static struct wlr_output_power_manager_v1 *power_mgr;
@ -115,28 +109,27 @@ index 8101ffa..4c3f12a 100644
static struct wlr_pointer_constraints_v1 *pointer_constraints;
static struct wlr_relative_pointer_manager_v1 *relative_pointer_mgr;
@@ -1088,6 +1095,20 @@ createmon(struct wl_listener *listener, void *data)
@@ -1084,6 +1090,19 @@ createmon(struct wl_listener *listener, void *data)
LISTEN(&wlr_output->events.destroy, &m->destroy, cleanupmon);
LISTEN(&wlr_output->events.request_state, &m->request_state, requestmonstate);
+ /* Enable HDR if supported and desired */
+ if ((r->image_desc) && (drw->features.output_color_transform) &&
+ (wlr_output->supported_primaries & r->image_desc->primaries) &&
+ (wlr_output->supported_transfer_functions & r->image_desc->transfer_function)) {
+ // Set framebuffer pixel format to one requested
+ if ((r->hdr) &&
+ (wlr_output->supported_primaries & WLR_COLOR_NAMED_PRIMARIES_BT2020) &&
+ (wlr_output->supported_transfer_functions & WLR_COLOR_TRANSFER_FUNCTION_ST2084_PQ) &&
+ (drw->features.output_color_transform)) {
+ const struct wlr_output_image_description image_desc = {
+ .primaries = WLR_COLOR_NAMED_PRIMARIES_BT2020,
+ .transfer_function = WLR_COLOR_TRANSFER_FUNCTION_ST2084_PQ,
+ };
+ wlr_output_state_set_render_format(&state, r->render_format);
+ // Check if unset and use default well-known primaries (white point will never be 0.0 if set)
+ struct wlr_color_primaries *prim = &r->image_desc->mastering_display_primaries;
+ if (prim->white.x == 0.0 && prim->white.y == 0.0) {
+ wlr_color_primaries_from_named (&r->image_desc->mastering_display_primaries, r->image_desc->primaries);
+ }
+ wlr_output_state_set_image_description(&state, r->image_desc);
+ wlr_output_state_set_image_description(&state, &image_desc);
+ }
+
wlr_output_state_set_enabled(&state, 1);
wlr_output_commit_state(wlr_output, &state);
wlr_output_state_finish(&state);
@@ -2653,6 +2674,36 @@ setup(void)
@@ -2645,6 +2664,37 @@ setup(void)
wl_signal_add(&output_mgr->events.apply, &output_mgr_apply);
wl_signal_add(&output_mgr->events.test, &output_mgr_test);
@ -150,29 +143,30 @@ index 8101ffa..4c3f12a 100644
+ .set_mastering_display_primaries = true,
+ };
+
+ color_mgr_options->render_intents_len = 1;
+ color_mgr_options->render_intents_len = 2;
+ enum wp_color_manager_v1_render_intent *render_intents = ecalloc(color_mgr_options->render_intents_len, sizeof(*render_intents));
+ render_intents[0] = WP_COLOR_MANAGER_V1_RENDER_INTENT_PERCEPTUAL;
+ render_intents[1] = WP_COLOR_MANAGER_V1_RENDER_INTENT_ABSOLUTE;
+ color_mgr_options->render_intents = render_intents;
+
+ enum wp_color_manager_v1_primaries *primaries = wlr_color_manager_v1_primaries_list_from_renderer(drw, &color_mgr_options->primaries_len);
+ enum wp_color_manager_v1_primaries *primaries = ecalloc(color_mgr_options->primaries_len, sizeof(*primaries));
+ primaries[0] = WP_COLOR_MANAGER_V1_PRIMARIES_SRGB;
+ primaries[1] = WP_COLOR_MANAGER_V1_PRIMARIES_BT2020;
+ color_mgr_options->primaries = primaries;
+
+ enum wp_color_manager_v1_transfer_function *transfer_functions = wlr_color_manager_v1_transfer_function_list_from_renderer(drw, &color_mgr_options->transfer_functions_len);
+ color_mgr_options->transfer_functions_len = 3;
+ enum wp_color_manager_v1_transfer_function *transfer_functions = ecalloc(color_mgr_options->transfer_functions_len, sizeof(*transfer_functions));
+ transfer_functions[0] = WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_SRGB;
+ transfer_functions[1] = WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_ST2084_PQ;
+ transfer_functions[2] = WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_EXT_LINEAR;
+ color_mgr_options->transfer_functions = transfer_functions;
+
+ color_mgr = wlr_color_manager_v1_create(dpy, 2, color_mgr_options);
+ wlr_scene_set_color_manager_v1(scene, color_mgr);
+
+ wlr_color_representation_manager_v1_create_with_renderer(dpy, 1, drw);
+
+ free(transfer_functions);
+ free(primaries);
+ color_mgr = wlr_color_manager_v1_create(dpy, 1, color_mgr_options);
+ }
+
/* Make sure XWayland clients don't connect to the parent X server,
* e.g when running in the x11 backend or the wayland backend and the
* compositor has Xwayland support */
--
2.54.0
2.51.0

View File

@ -1,12 +1,12 @@
From c4a9254ef63bbe1bae9ed525250e0b187668397f Mon Sep 17 00:00:00 2001
From 34c613a77683ab0e2beac36276315d037ba222a5 Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?Andr=C3=A9=20Desgualdo=20Pereira?= <desgua@gmail.com>
Date: Fri, 1 May 2026 08:17:28 -0300
Subject: [PATCH] swapfocus patch improvement
Date: Thu, 30 Apr 2026 13:36:03 -0300
Subject: [PATCH] add swapfocus patch
---
config.def.h | 1 +
dwl.c | 47 +++++++++++++++++++++++++++++++++++++++++++++++
2 files changed, 48 insertions(+)
dwl.c | 45 +++++++++++++++++++++++++++++++++++++++++++++
2 files changed, 46 insertions(+)
diff --git a/config.def.h b/config.def.h
index 8a6eda0..23e502d 100644
@ -21,7 +21,7 @@ index 8a6eda0..23e502d 100644
{ MODKEY, XKB_KEY_f, setlayout, {.v = &layouts[1]} },
{ MODKEY, XKB_KEY_m, setlayout, {.v = &layouts[2]} },
diff --git a/dwl.c b/dwl.c
index 101a45f..5e78719 100644
index 101a45f..dac34a6 100644
--- a/dwl.c
+++ b/dwl.c
@@ -332,6 +332,7 @@ static void setsel(struct wl_listener *listener, void *data);
@ -61,7 +61,7 @@ index 101a45f..5e78719 100644
/* Put the new client atop the focus stack and select its monitor */
if (c && !client_is_unmanaged(c)) {
wl_list_remove(&c->flink);
@@ -2679,6 +2688,44 @@ spawn(const Arg *arg)
@@ -2679,6 +2688,42 @@ spawn(const Arg *arg)
}
}
@ -71,15 +71,16 @@ index 101a45f..5e78719 100644
+ Client *c;
+ int found = 0;
+
+ if (!prevclient)
+ return;
+
+ /* Verify the client still exists in the list of managed windows */
+ if (prevclient) {
+ wl_list_for_each(c, &clients, link) {
+ if (c == prevclient) {
+ found = 1;
+ break;
+ }
+ }
+ }
+
+ if (found && !client_is_unmanaged(prevclient)) {
+ /* Get the bitmask of currently visible tags on the prevclient's monitor */
@ -97,9 +98,6 @@ index 101a45f..5e78719 100644
+ /* Tag IS visible: Just swap focus within the same view */
+ focusclient(prevclient, 1);
+ }
+ } else {
+ Arg a = {.ui = 0};
+ view(&a);
+ }
+}
+
diff --git a/config.def.h b/config.def.h
index 8a6eda0..d203edc 100644
index 95c2afa..43af85b 100644
--- a/config.def.h
+++ b/config.def.h
@@ -35,15 +35,31 @@ static const Layout layouts[] = {
{ "[M]", monocle },
};
+
+/*
+static struct wlr_output_image_description generic_bt2020_pq = {
+ .primaries = WLR_COLOR_NAMED_PRIMARIES_BT2020,
+ .transfer_function = WLR_COLOR_TRANSFER_FUNCTION_ST2084_PQ,
+ .max_cll = 10000,
+ .max_fall = 400,
+ .mastering_luminance = {
+ .min = 0,
+ .max = 10000,
+ },
+};
+*/
+
/* monitors */
/* (x=-1, y=-1) is reserved as an "autoconfigure" monitor position indicator
* WARNING: negative values other than (-1, -1) cause problems with Xwayland clients due to
* https://gitlab.freedesktop.org/xorg/xserver/-/issues/899 */
@@ -43,12 +43,14 @@ static const Layout layouts[] = {
*/
/* NOTE: ALWAYS add a fallback rule, even if you are completely sure it won't be used */
static const MonitorRule monrules[] = {
- /* name mfact nmaster scale layout rotate/reflect x y
- * example of a HiDPI laptop monitor:
- { "eDP-1", 0.5f, 1, 2, &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL, -1, -1 }, */
- { NULL, 0.55f, 1, 1, &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL, -1, -1 },
+ /* name mfact nmaster scale layout rotate/reflect x y image_desc pixel_format
+ * example of a HiDPI laptop monitor:
+ { "eDP-1", 0.5f, 1, 2, &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL, -1, -1, NULL, DRM_FORMAT_XRGB8888 },
- /* name mfact nmaster scale layout rotate/reflect x y */
+ /* name mfact nmaster scale layout rotate/reflect x y hdr render_format */
/* example of a HiDPI laptop monitor:
- { "eDP-1", 0.5f, 1, 2, &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL, -1, -1 },
+ { "eDP-1", 0.5f, 1, 2, &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL, -1, -1, 0, DRM_FORMAT_XRGB8888 },
+ * HDR Example
+ { NULL, 0.5f, 1, 1, &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL, -1, -1, &generic_bt2020_pq, DRM_FORMAT_XRGB2101010 }, */
+ { NULL, 0.55f, 1, 1, &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL, -1, -1, NULL, DRM_FORMAT_XRGB8888 },
/* default monitor rule: can be changed but cannot be eliminated; at least one monitor rule must exist */
+ { NULL, 0.55f, 1, 1, &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL, -1, -1, 1, DRM_FORMAT_XRGB2101010 },
*/
/* defaults */
- { NULL, 0.55f, 1, 1, &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL, -1, -1 },
+ { NULL, 0.55f, 1, 1, &layouts[0], WL_OUTPUT_TRANSFORM_NORMAL, -1, -1, 0, DRM_FORMAT_XRGB8888 },
};
/* keyboard */